/*************************************************************************************************
 * Implementation of wavermod
 *                                                      Copyright (C) 2004-2006 Mikio Hirabayashi
 * This file is part of Hyper Estraier.
 * Hyper Estraier is free software; you can redistribute it and/or modify it under the terms of
 * the GNU Lesser General Public License as published by the Free Software Foundation; either
 * version 2.1 of the License or any later version.  Hyper Estraier is distributed in the hope
 * that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 * License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with Hyper
 * Estraier; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA.
 *************************************************************************************************/


#include "wavermod.h"


/* private function prototypes */
static int queue_compare(const char *aptr, int asiz, const char *bptr, int bsiz);



/*************************************************************************************************
 * pseudo API
 *************************************************************************************************/


/* Open a priority queue. */
QUEUE *queue_open(const char *name){
  QUEUE *queue;
  VILLA *db;
  const char *vbuf;
  int vomode, vsiz;
  double max;
  assert(name);
  vomode = VL_OWRITER | VL_OCREAT;
  if(ESTUSEBZIP){
    vomode |= VL_OXCOMP;
  } else if(ESTUSELZO){
    vomode |= VL_OYCOMP;
  } else if(ESTUSEZLIB){
    vomode |= VL_OZCOMP;
  }
  if(!(db = vlopen(name, vomode, queue_compare))) return NULL;
  max = 1.0;
  vlcurlast(db);
  if((vbuf = vlcurkeycache(db, &vsiz)) != NULL && vsiz == sizeof(double))
    max = *(double *)vbuf;
  if(max < 1.0) max = 1.0;
  queue = cbmalloc(sizeof(QUEUE));
  queue->db = db;
  queue->max = max;
  return queue;
}


/* Close a priority queue. */
int queue_close(QUEUE *queue){
  int err;
  assert(queue);
  err = FALSE;
  if(!vlclose(queue->db)) err = TRUE;
  free(queue);
  return err ? FALSE : TRUE;
}


/* Spread the priority space of a priority queue. */
void queue_spread(QUEUE *queue, double inc){
  assert(queue);
  queue->max += inc;
  if(queue->max < 1.0) queue->max = 1.0;
}


/* Enqueue a record into a priority queue. */
int queue_enqueue(QUEUE *queue, const char *str, double priority){
  int err;
  assert(queue && str);
  if(priority < 0.0) priority = 0.0;
  if(priority > 1.0) priority = 1.0;
  priority *= queue->max;
  err = FALSE;
  if(!vlput(queue->db, (char *)&priority, sizeof(double), str, -1, VL_DDUP)) err = TRUE;
  return err ? FALSE : TRUE;
}


/* Dequeue a record from a priority queue. */
char *queue_dequeue(QUEUE *queue){
  char *vbuf;
  assert(queue);
  vlcurfirst(queue->db);
  if(!(vbuf = vlcurval(queue->db, NULL))) return NULL;
  vlcurout(queue->db);
  return vbuf;
}



/*************************************************************************************************
 * private objects
 *************************************************************************************************/


/* Compare keys of two records as double type objects.
   `aptr' specifies the pointer to the region of one key.
   `asiz' specifies the size of the region of one key.
   `bptr' specifies the pointer to the region of the other key.
   `bsiz' specifies the size of the region of the other key.
   The return value is positive if the former is big, negative if the latter is big, 0 if both
   are equivalent. */
static int queue_compare(const char *aptr, int asiz, const char *bptr, int bsiz){
  double anum, bnum;
  assert(aptr && asiz >= 0 && bptr && bsiz >= 0);
  if(asiz != bsiz) return asiz - bsiz;
  anum = (asiz == sizeof(double) ? *(double *)aptr : INT_MIN);
  bnum = (bsiz == sizeof(double) ? *(double *)bptr : INT_MIN);
  return (int)(anum - bnum);
}



/* END OF FILE */
